{% extends "admin/base.html" %}
{% load url from future %}

{% block extrahead %}
    <link rel="stylesheet" href="{{STATIC_URL}}css/cms.pages.css"/>
    {{block.super}}
{% endblock %}


{% block title %}{{title}} - {{settings.SITE_NAME}} administration{% endblock %}


{% block branding %}
    <h1 id="site-name">{{settings.SITE_NAME}} administration</h1>
{% endblock %}


{% block footer_links %}
  <a href="https://onespacemedia.zendesk.com/anonymous_requests/new" target="_blank" class="icon"><i class="icon-question-sign"></i>Support</a>
{% endblock %}

{% block copyright %}
  Copyright &copy; {% now "Y" %} {{settings.SITE_NAME}}<br>Developed by <a href="http://onespacemedia.com" target="_blank">Onespacemedia</a>
{% endblock %}

{% block footer_branding %}
{% endblock %}
